/*******************************************************************************
* Title                 :   Pulse click Demonstration
* Filename              :   PulseExampleSTM32.c
* Author                :   RBL
* Origin Date           :   15/12/2015
* Notes                 :   None
*******************************************************************************/
/*************** MODULE REVISION LOG ******************************************
*
*    Date    Software Version    Initials   Description
*  15/12/15       .1              RBL       Module Created.
*  16/12/15       .2              RBL       Correction of Frequency
*
*******************************************************************************/
/**
 * @file PulseExampleSTM32.c
 * @brief Example of Pulse click operation.
 *
 * @note Pulse board is configured as default with output pin to the analog
 * pin on MikroBus 1.
 */
/******************************************************************************
* Includes
*******************************************************************************/
#include <stdint.h>
#include <stdbool.h>
#include "built_in.h"
#include "resources.h"

/******************************************************************************
* Module Preprocessor Constants
*******************************************************************************/

/******************************************************************************
* Module Preprocessor Macros
*******************************************************************************/

/******************************************************************************
* Module Typedefs
*******************************************************************************/

/******************************************************************************
* Module Variable Definitions
*******************************************************************************/
// TFT module connections
unsigned int TFT_DataPort at GPIOE_ODR;
sbit TFT_RST at GPIOE_ODR.B8;
sbit TFT_RS at GPIOE_ODR.B12;
sbit TFT_CS at GPIOE_ODR.B15;
sbit TFT_RD at GPIOE_ODR.B10;
sbit TFT_WR at GPIOE_ODR.B11;
sbit TFT_BLED at GPIOE_ODR.B9;
// End TFT module connections

volatile double pulse_count;
volatile double frequency;
volatile bool update_time;

/******************************************************************************
* Function Prototypes
*******************************************************************************/
void init_timer_a( void );
void system_init( void );
void auxiliary_code( void );
void set_index( unsigned short index );
void write_command( unsigned short cmd );
void write_data( unsigned int _data );
void tft_initialize( void );
void tft_update( void );

/******************************************************************************
* Function Definitions
*******************************************************************************/
void init_timer_a()
{
    RCC_APB1ENR.TIM2EN = 1;
    TIM2_CR1.CEN = 0;
    TIM2_PSC = 1439;
    TIM2_ARR = 62499;
    NVIC_IntEnable( IVT_INT_TIM2 );
    TIM2_DIER.UIE = 1;
    TIM2_CR1.CEN = 1;
}

void tft_initialize()
{
    TFT_Set_Pen( CL_WHITE, 1 );
    TFT_Set_Brush( 1, CL_WHITE, 0, 0, 0, 0 );
    TFT_Set_Font( TFT_defaultFont, CL_BLACK, FO_HORIZONTAL );

    TFT_Fill_Screen( CL_WHITE );
    TFT_Set_Pen( CL_BLACK, 1 );
    TFT_Line( 20, 220, 300, 220 );
    TFT_Line( 20,  46, 300,  46 );
    TFT_Set_Font( &HandelGothic_BT21x22_Regular, CL_RED, FO_HORIZONTAL );
    TFT_Write_Text( "Pulse click", 110, 14 );
    TFT_Set_Font( &Verdana12x13_Regular, CL_BLACK, FO_HORIZONTAL );
    TFT_Write_Text( "EasyMx PRO v7 for STM32", 19, 223 );
    TFT_Set_Font( &Verdana12x13_Regular, CL_RED, FO_HORIZONTAL );
    TFT_Write_Text( "www.mikroe.com", 200, 223 );
    TFT_Set_Font( &HandelGothic_BT21x22_Regular, CL_RED, FO_HORIZONTAL );
    TFT_Write_Text( "Frequency", 110, 100 );
}

void tft_update()
{
    char tmp_txt[20];
    static double last_frequency;

    TFT_Set_Font( &HandelGothic_BT21x22_Regular, CL_WHITE, FO_HORIZONTAL );
    sprintf( tmp_txt, "%3.2fHz", last_frequency );
    TFT_Write_Text( tmp_txt, 130, 130 );
    TFT_Set_Font( &HandelGothic_BT21x22_Regular, CL_BLACK, FO_HORIZONTAL );
    sprintf( tmp_txt, "%3.2fHz", frequency );
    TFT_Write_Text( tmp_txt, 130, 130 );
    last_frequency = frequency;

    update_time = false;
}

void system_init()
{
    DisableInterrupts();
    // Pulse Pin setup
    GPIO_Config( &GPIOA_BASE, _GPIO_PINMASK_4, _GPIO_CFG_DIGITAL_INPUT );

    RCC_APB2ENR.AFIOEN = 1;         // Enable clock for alternate pin functions

    EXTI_RTSR |= ( 1 << TR4 );      // Set interrupt on Rising edge
    EXTI_IMR |= ( 1 << MR4 );       // Set mask
    NVIC_IntEnable( IVT_INT_EXTI4 );// Enable External interrupt

    // TFT
    TFT_Init_ILI9341_8bit( 320, 240 );
    tft_initialize();
    // Timer
    init_timer_a();
    pulse_count = 0;

    EnableInterrupts();             // Enables the processor interrupt.
}

void main()
{
    system_init();

    while( 1 )
    {
        if( update_time )
            tft_update();
    }
}

void pulse_ISR() iv IVT_INT_EXTI4 ics ICS_AUTO
{
    EXTI_PR |= ( 1 << PR4 );
    pulse_count += 1.0f;
}

void timer2_interrupt() iv IVT_INT_TIM2
{
    TIM2_SR.UIF = 0;
    frequency = pulse_count * .075f ;
    pulse_count = 0;
    update_time = true;
}
/*************** END OF FUNCTIONS ***************************************************************************/